/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(log, _, Promise, injector) {

    /**
     * Safely execute all hooks (sorted by priority) for the given operation. Hooks may modify the 
     * operation or the result (pre or post)
     */
    return function(phase, key, operation) {

        let hookKey = 'hook$_' + key + "-" + phase;

        log.debug("Resolving all hooks for key", hookKey);

        // Retrieve all hooks for the given key and phase
        let hooks = injector.resolve(hookKey, {
            list: true
        });

        return Promise.reduce(_.sortBy(hooks || [], "priority"), function(operation, hook) {
            hook.phase = phase;
            hook.key = key;
            return hook.hook(operation);
        }, operation).then(function() {
            return operation;
        });
    };
};