/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(service$httpQuery, service$_mongo, service$executeHook, service$pageLinkBuilder) {

    let resource = this;

    return function(req, res) {
        let filter = service$httpQuery(req.query, resource.schema);
        let opts = {},
            resultPromise;

        if (req.query.limit) {
            opts.limit = parseInt(req.query.limit);
        }

        if (req.query.offset) {
            opts.offset = parseInt(req.query.offset);
        }

        if (req.query.meta) {
            opts.meta = !!req.query.meta;
        }

        opts.type = filter.opts.type;
        opts.sort = filter.sort;
        opts.view = resource.view || 'public';
        opts.schema = resource.schema;

        let operation = {
            resource: resource,
            query: filter.query,
            options: opts,
            req: req,
            res: res
        };

        if (filter.op === 'find') {
            resultPromise = service$executeHook('pre:list', resource.key, operation).then(function(operation) {
                return service$_mongo.list(operation.resource.db, operation.resource.collection, operation.query, operation.options);
            });
        } else if (filter.op === 'search') {
            resultPromise = service$executeHook('pre:list', resource.key, operation).then(function(operation) {
                return service$_mongo.search(operation.resource.db, operation.resource.collection, operation.query, operation.options);
            });
        } else {
            throw new Error('Invalid operation');
        }

        return resultPromise.then(function(result) {
            operation.result = result;
            return service$executeHook('post:list', resource.key, operation);
        }).then(function(operation) {
            if (operation.options.meta) {
                operation.result.meta.pages = service$pageLinkBuilder(req.originalUrl, operation.result.meta.total, operation.result.meta.page, operation.result.meta.limit);
            }
            return res.json(operation.result);
        });
    };

};