/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
module.exports = function(log, service$_mongo, service$executeHook) {

    let resource = this;

    return function(req, res) {
        log.debug("UPDATE RESOURCE %s:%s", resource.key, req.params.id);

        let selector = {};
        selector[resource.idField || '_id'] = req.params.id;

        let operation = {
            resource: resource,
            selector: selector,
            data: req.body,
            options: {
                multi: req.query.multi,
                upsert: req.query.upsert,
                view: resource.view || 'public',
                schema: resource.schema
            },
            req: req,
            res: res
        };

        return service$executeHook('pre:update', resource.key, operation).then(function(operation) {
            return service$_mongo.update(operation.resource.db, operation.resource.collection, operation.selector, operation.data, operation.options).then(function(result) {
                operation.result = result;
                return service$executeHook('post:update', resource.key, operation);
            }).then(function(operation) {
                return res.json(operation.result);
            });
        }).catch(function(err) {
            if (err.isJoi) {
                res.status(400).send(err);
            } else {
                throw err;
            }
        });
    };

};