/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */
module.exports = function() {

    const URL = require('url'),
        qs = require('querystring');

    return function(baseUrl, total, page, pageSize) {

        // Pre-process the received url, remove any offset and limit parameters
        let urlComponents = URL.parse(baseUrl);
        let query = qs.parse(urlComponents.query);

        delete query.limit;
        delete query.offset;

        urlComponents.search = qs.stringify(query);
        baseUrl = URL.format(urlComponents);

        let paramFlag = baseUrl.indexOf('?') !== -1 ? '&' : "?";

        let lastPageNo = Math.floor(total / pageSize);

        if (total % pageSize > 0) {
            lastPageNo++;
        }

        let result = {
            first: baseUrl + paramFlag + `offset=0&limit=${pageSize}`,
            last: baseUrl + paramFlag + `offset=${lastPageNo * pageSize}&limit=${pageSize}`,
            pages: []
        };

        // Compute all pages
        for (let i = 0; i < lastPageNo; i++) {
            let url = baseUrl + paramFlag + `offset=${i * pageSize}&limit=${pageSize}`;
            result.pages.push(url);
        }

        return result;
    };

};