/*
 Copyright 2016 Pagemesh Inc.

 Licensed under the Apache License, Version 2.0 (the "License");
 you may not use this file except in compliance with the License.
 You may obtain a copy of the License at

 http://www.apache.org/licenses/LICENSE-2.0

 Unless required by applicable law or agreed to in writing, software
 distributed under the License is distributed on an "AS IS" BASIS,
 WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 See the License for the specific language governing permissions and
 limitations under the License.
 */

module.exports = function(log, Ximera, Promise, glob, injector, _, db$_, config) {
    log.debug("Loading and registering all rest resources");

    const Path = require('path');

    // Register our Resource implementation
    injector.register('ResourceContainer', require('./rest-resource-container'));
    injector.register('Resource', require('./rest-resource'));

    // Register routers and hooks
    return Promise.join(
        Ximera.findComponentFiles('*.router.js').then(function(files) {
            return Promise.map(files, function(file) {
                log.debug("Registering router %s", file);
                let key = Ximera.makeKeyFromFile(file, 'router');
                let cfg = _.get(config.routers, key) || {};
                let factory = require(Path.resolve(file));
                return injector.execute(factory, {
                    config: cfg
                });
            });
        }),
        Ximera.findComponentFiles('*.hook.js').then(function(files) {
            return Promise.map(files, function(file) {
                log.debug("Registering REST operation hook %s", file);
                let key = Ximera.makeKeyFromFile(file, 'hook');
                let cfg = _.get(config.hooks, key) || {};
                let factory = require(Path.resolve(file));
                return injector.execute(factory, {
                    config: cfg
                });
            });
        })
    );
};